package cheddar.aadl;

import java.util.ArrayList;
import java.util.List;

import org.osate.aadl2.EnumerationLiteral;
import org.osate.aadl2.NamedElement;
import org.osate.aadl2.Property;
import org.osate.aadl2.PropertyExpression;
import org.osate.aadl2.StringLiteral;
import org.osate.aadl2.UnitLiteral;
import org.osate.aadl2.properties.PropertyLookupException;
import org.osate.xtext.aadl2.properties.util.AadlProject;
import org.osate.xtext.aadl2.properties.util.GetProperties;
import org.osate.xtext.aadl2.properties.util.PropertyUtils;

public class Cheddar_Properties_Util {

	public static long getFixedPriority(final NamedElement ne) {
		Property priority = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.FIXED_PRIORITY);
		return PropertyUtils.getIntegerValue(ne, priority, -1);
	}

	public static Integer getScheduler_Quantum(final NamedElement ne) {
		Property scheduler_quantum = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.SCHEDULER_QUANTUM);
		return new Integer((int) PropertyUtils.getIntegerValue(ne, scheduler_quantum));
	}

	public static boolean getPreemptive_Scheduler(final NamedElement ne) {
		Property preemptive_scheduler = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.PREEMPTIVE_SCHEDULER);
		return PropertyUtils.getBooleanValue(ne, preemptive_scheduler);
	}

	public static long getDataConcurrencyState(final NamedElement ne) {
		Property data_concurrency_state = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DATA_CONCURRENCY_STATE);
		return PropertyUtils.getIntegerValue(ne, data_concurrency_state, 0);
	}

	public static boolean getIs_Preemptive(final NamedElement ne) {
		Property is_preemptive = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.IS_PREEMPTIBLE);
		return PropertyUtils.getBooleanValue(ne, is_preemptive);
	}

	public static boolean getDispatch_Seed_Is_Predictable(final NamedElement ne) {
		Property Dispatch_Seed_Is_Predictable = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_SEED_IS_PREDICTABLE);
		return PropertyUtils.getBooleanValue(ne, Dispatch_Seed_Is_Predictable);
	}

	public static long getDispatch_Seed_Value(final NamedElement ne) {
		Property Dispatch_Seed_Value = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_SEED_VALUE);
		return PropertyUtils.getIntegerValue(ne, Dispatch_Seed_Value, 0);
	}

	public static String getSource_Text(final NamedElement ne) {
		Property Source_Text = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.SOURCE_TEXT);
		return PropertyUtils.getStringValue(ne, Source_Text);
	}

	public static long getCriticality(final NamedElement ne) {
		Property Criticality = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.CRITICALITY);
		return PropertyUtils.getIntegerValue(ne, Criticality, 0);
	}

	public static double getSource_Global_Text_Size(final NamedElement ne) {
		Property Source_Global_Text_Size = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.SOURCE_GLOBAL_TEXT_SIZE);
		UnitLiteral Bytes = GetProperties.findUnitLiteral(Source_Global_Text_Size, AadlProject.B_LITERAL);
		return GetProperties.getDataSize(ne, Bytes);
	}

	public static double getSource_Global_Stack_Size(final NamedElement ne) {
		Property Source_Global_Stack_Size = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.SOURCE_GLOBAL_STACK_SIZE);
		UnitLiteral Bytes = GetProperties.findUnitLiteral(Source_Global_Stack_Size, AadlProject.B_LITERAL);
		return GetProperties.getDataSize(ne, Bytes);
	}

	public static double getSource_Global_Data_Size(final NamedElement ne) {
		Property Source_Global_Data_Size = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.SOURCE_GLOBAL_DATA_SIZE);
		UnitLiteral Bytes = GetProperties.findUnitLiteral(Source_Global_Data_Size, AadlProject.B_LITERAL);
		return GetProperties.getDataSize(ne, Bytes);
	}

	public static EnumerationLiteral getPOSIX_Scheduling_Policy(final NamedElement ne) {
		try {
			Property POSIX_Scheduling_Policy = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
					Cheddar_Properties.POSIX_SCHEDULING_POLICY);
			return PropertyUtils.getEnumLiteral(ne, POSIX_Scheduling_Policy);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static long getDispatch_Offset_Value_k(final NamedElement ne) {
		Property Dispatch_Offset_Value_k = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_OFFSET_VALUE_K);
		return PropertyUtils.getIntegerValue(ne, Dispatch_Offset_Value_k, 0);
	}

	public static double getBound_On_Data_Blocking_TimeinMilliSec(final NamedElement ne) {
		Property Bound_On_Data_Blocking_Time = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.BOUND_ON_DATA_BLOCKING_TIME);
		UnitLiteral milliSecond = GetProperties.findUnitLiteral(Bound_On_Data_Blocking_Time, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledNumberValue(ne, Bound_On_Data_Blocking_Time, milliSecond, 0.0);
	}

	public static double getDispatch_Absolute_TimeinMilliSec(final NamedElement ne) {
		Property Dispatch_Absolute_Time = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_ABSOLUTE_TIME);
		UnitLiteral milliSecond = GetProperties.findUnitLiteral(Dispatch_Absolute_Time, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledNumberValue(ne, Dispatch_Absolute_Time, milliSecond, 0.0);
	}

	public static double getDispatch_JitterinMilliSec(final NamedElement ne) {
		Property Dispatch_Jitter = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_JITTER);
		UnitLiteral milliSecond = GetProperties.findUnitLiteral(Dispatch_Jitter, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledNumberValue(ne, Dispatch_Jitter, milliSecond, 0.0);
	}

	public static double getDispatch_Offset_Time_kinMilliSec(final NamedElement ne) {
		Property Dispatch_Offset_Time_k = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
				Cheddar_Properties.DISPATCH_OFFSET_TIME_K);
		UnitLiteral milliSecond = GetProperties.findUnitLiteral(Dispatch_Offset_Time_k, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledNumberValue(ne, Dispatch_Offset_Time_k, milliSecond, 0.0);
	}

	public static List<String> getDataEnumerators(final NamedElement ne) {
		List<String> res;
		res = new ArrayList<String>();
		try {
			Property Critical_Section = GetProperties.lookupPropertyDefinition(ne, Cheddar_Properties._NAME,
					Cheddar_Properties.CRITICAL_SECTION);

			List<? extends PropertyExpression> propertyValues = ne.getPropertyValueList(Critical_Section);
			for (PropertyExpression propertyExpression : propertyValues) {
				StringLiteral sl = (StringLiteral) propertyExpression;
				res.add(sl.getValue());
			}
			return res;

		} catch (PropertyLookupException e) {
			return null;
		}
	}
}
