package cheddar.aadl;

import org.osate.aadl2.instance.ComponentInstance;
import org.osate.aadl2.instance.SystemInstance;

import cheddar.adl.Address_Space;
import cheddar.adl.Core_Unit;
import cheddar.adl.Generic_Resource;
import cheddar.adl.Generic_Task;


public class To_System {

	// Cheddar entities to produce
	//
	public cheddar.adl.System result;

	// Cheddar unique object identifier
	//
	Cheddar_Id_Generator id;


	public To_System() {
		id = new Cheddar_Id_Generator();
	}


	// Entry point to transform AADL => Cheddar ADL
	//
	public String parse(SystemInstance si) {

		result = new cheddar.adl.System();

		// First, we create Cheddar ADL processor units
		// We have two possibilities:
		// - AADL processor units that lead to Cheddar Monoprocessor
		// - AADL systems modeling MPSOC that lead to Cheddar Multicore processor
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case SYSTEM:
				if (To_Processor.Is_MPSoC_Component(component)) {
					To_Processor.Create_MPSoC_Processor_Component(result, id, component);
				}
				// ComponentClassifier implemented_as = Properties_With_Default_Value.getImplementedAs(component);
				// System.out.println("implemented_as = " + implemented_as.getName() + " name = " + component.getName());

				break;
			case PROCESSOR:
				ComponentInstance parent = component.getContainingComponentInstance();
				if (!To_Processor.Is_MPSoC_Component(parent)) {
					Core_Unit c = To_Core_Unit.Create_Core_Unit(result, id, component);
					To_Processor.Create_MonoCore_Processor(result, id, c, component);
				}
				break;
			default:
				break;
			}

		}

		// Create Cheddar ADL address spaces
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case PROCESS:
				To_Address_Space.Create_Process_Component(result, id, component);
				break;
			default:
				break;
			}
		}

		// Create Cheddar ADL tasks and resources
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case THREAD:
				To_Task.Create_Thread_Component(result, id, component);
				break;
			case DATA:
				To_Resource.Create_Data_Component(result, id, component);
				break;
			default:
				break;
			}
		}


		// Create dependencies between Cheddar ADL instances
		//
		for (ComponentInstance component : si.getAllComponentInstances()) {
			switch (component.getCategory()) {
			case THREAD:
				To_Dependency.Thread_Component_Dependencies(result, id, component);
				break;
			default:
				break;
			}
		}


		// Depending on how bindings are expressed, we may have
		// Address_Space component with unassigned "cpu_name"
		// Fix unassigned "cpu_name"
		if (!result.address_spaces.isEmpty()) {
			for (Address_Space an_address_space : result.address_spaces) {
				if (an_address_space.getCpuName().equals(""))
				{
					if (!result.tasks.isEmpty()) {
						for (Generic_Task a_task : result.tasks) {
							if (a_task.getAddressSpaceName().equals(an_address_space.getName())) {
								an_address_space.setCpuName(a_task.getCpuName());
							}
						}
					}
					if (!result.resources.isEmpty()) {
						for (Generic_Resource a_resource : result.resources) {
							if (a_resource.getAddressSpaceName().equals(an_address_space.getName())) {
								an_address_space.setCpuName(a_resource.getCpuName());
							}
						}
					}
				}
			}
		}

		// Produce Cheddar XML V3 and return it
		//
		return result.xmlString();
	}




}
