package cheddar.aadl;

import org.osate.aadl2.instance.ComponentInstance;

import cheddar.adl.Aperiodic_Task;
import cheddar.adl.Generic_Task;
import cheddar.adl.Objects_Type;
import cheddar.adl.Periodic_Task;
import cheddar.adl.Poisson_Task;
import cheddar.adl.Policies;
import cheddar.adl.Sporadic_Task;
import cheddar.adl.Tasks_Type;
import cheddar.osate2_plugin.Debug;

public class To_Task {


	static public void Create_Thread_Component(cheddar.adl.System result, Cheddar_Id_Generator id,
			ComponentInstance component) {

		String name = component.getFullName().toString();

		Integer priority = Properties_With_Default_Value.getPriority(component);
		Integer deadline = Properties_With_Default_Value.getDeadline(component);
		Integer period = Properties_With_Default_Value.getPeriod(component);
		Integer capacity = Properties_With_Default_Value.getExecutionTime(component);
		Integer jitter = Properties_With_Default_Value.getDispatch_Jitter(component);
		Integer start_time = Properties_With_Default_Value.getDispatch_Absolute_Time(component);
		Policies policy = Properties_With_Default_Value.getPOSIX_Scheduling_Policy(component);

		String address_space_name = Properties_With_Default_Value.getBoundProcess(component);
		String cpu_name = Properties_With_Default_Value.getBoundProcessor(component);


		Tasks_Type tt = Properties_With_Default_Value.getDispatchProtocol(component);

		Generic_Task gt;

		switch (tt) {
		case Aperiodic_Type:
			Aperiodic_Task at = new Aperiodic_Task();
			gt = at;
			break;
		case Sporadic_Type:
			Sporadic_Task st = new Sporadic_Task();
			st.setPeriod(period);
			st.setJitter(jitter);
			gt = st;
			break;
		case Poisson_Type:
			Poisson_Task pt = new Poisson_Task();
			pt.setPeriod(period);
			pt.setJitter(jitter);
			gt = pt;
			break;
		// By default, tasks are periodic
		default:
			Periodic_Task pt2 = new Periodic_Task();
			pt2.setPeriod(period);
			pt2.setJitter(jitter);
			gt = pt2;
			break;
		}

		gt.setcheddarXmlId(id.getAndIncrement());
		gt.setName(name);

		gt.setAddressSpaceName(address_space_name);
		gt.setCpuName(cpu_name);
		gt.setObjectType(Objects_Type.Task_Object_Type);

		gt.setCapacity(capacity);
		gt.setDeadline(deadline);
		gt.setPolicy(policy);
		gt.setPriority(priority);
		gt.setStartTime(start_time);

		Debug.Put("add task " + gt.getName());

		result.tasks.add(gt);
	}

}



