package cheddar.aadltests;

import java.util.HashMap;

import cheddar.adl.Address_Space;
import cheddar.adl.Core_Unit;
import cheddar.adl.Critical_Section;
import cheddar.adl.Migrations_Type;
import cheddar.adl.Mono_Core_Processor;
import cheddar.adl.Pcp_Resource;
import cheddar.adl.Periodic_Task;
import cheddar.adl.Preemptives_Type;
import cheddar.adl.Resources_Type;
import cheddar.adl.Schedulers_Type;
import cheddar.adl.Scheduling_Parameters;
import cheddar.core.XML_Pretty_Printer;


public class Test_XML {

	public static void main (String[] args){

		// New Cheddar system
		cheddar.adl.System s = new cheddar.adl.System();

		// A core unit, with its scheduling data
		Scheduling_Parameters p = new Scheduling_Parameters();
		p.setPreemptiveType(Preemptives_Type.Preemptive);
		p.setSchedulerType(Schedulers_Type.Rate_Monotonic_Protocol);

		Core_Unit c = new Core_Unit();
		c.setName("core1");
		c.setScheduling(p);
		c.setcheddarXmlId("1");

		s.cores.add(c);

		// A processor, with its core unit
		Mono_Core_Processor mcp = new Mono_Core_Processor();
		mcp.setName("cpu1");
		mcp.setcheddarXmlId("2");
		mcp.setCore(c);
		mcp.setMigrationType(Migrations_Type.No_Migration_Type);

		s.processors.add(mcp);

		// An address space
		Address_Space add = new Address_Space();
		add.setcheddarXmlId("3");
		add.setName("add1");
		add.setCpuName("cpu1");
		Scheduling_Parameters paddr = new Scheduling_Parameters();
		add.setScheduling(paddr);
		s.address_spaces.add(add);

		// Two periodic tasks
		Periodic_Task pt = new Periodic_Task();
		pt.setName("T1");
		pt.setcheddarXmlId("4");
		pt.setAddressSpaceName("add1");
		pt.setCapacity(3);
		pt.setCpuName("cpu1");
		pt.setDeadline(10);
		pt.setPeriod(10);
		pt.setPriority(100);
		s.tasks.add(pt);

		pt = new Periodic_Task();
		pt.setName("T2");
		pt.setcheddarXmlId("5");
		pt.setAddressSpaceName("add1");
		pt.setCapacity(13);
		pt.setCpuName("cpu1");
		pt.setDeadline(30);
		pt.setPeriod(30);
		pt.setPriority(50);
		s.tasks.add(pt);

		// A PCP resource shared between the two tasks
		Pcp_Resource r = new Pcp_Resource();
		r.setName("R1");
		r.setcheddarXmlId("6");
		r.setAddressSpaceName("add1");
		r.setCpuName("cpu1");
		r.setProtocol(Resources_Type.Priority_Ceiling_Protocol);
		r.setState(1);
		HashMap<String, Critical_Section> cs = new HashMap<String, Critical_Section>();
		Critical_Section scT1 = new Critical_Section();
		Critical_Section scT2 = new Critical_Section();
		scT1.setTaskBegin(1);
		scT1.setTaskEnd(2);
		scT2.setTaskBegin(1);
		scT2.setTaskEnd(3);
		cs.put("T1", scT1);
		cs.put("T2", scT2);
		r.setCriticalSections(cs);
		s.resources.add(r);

		// Produce XML file
		String result = s.xmlString();
		String formatted = XML_Pretty_Printer.Pretty_Print(result);
		System.out.println(formatted);
}

}
